﻿using MitMVVM.ViewModel;
using Rechnen.Model;  // Punkt 9.2
using System;
using System.Collections.Generic;
using System.ComponentModel;  // Punkt 9.1
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Windows.Media;

namespace Rechnen.ViewModel
{

    public class RechnenViewModel : INotifyPropertyChanged // Punkt 9.1
    {
        // Punkt 9.2, das Model
        Zahlen _z = new Zahlen();

        // Punkt 9.1 Schnittstellen-Ereignis -->
        public event PropertyChangedEventHandler PropertyChanged;
        protected internal void OnPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyname));
        }
        // <--

        #region Eigenschaften

        // Punkt 9.3 -->
        private string _zahl1;
        public string Zahl1
        {
            get { return this._zahl1; }
            set
            {
                this._zahl1 = value;
                // eine Änderung der Oberfläche veranlassen,
                // ist nur notwendig, wenn Sie die Felder
                // vom ViewModel aus auf 0 setzten wollen
                this.OnPropertyChanged("Zahl1");
            }
        }

        private string _zahl2;
        public string Zahl2
        {
            get { return this._zahl2; }
            set
            {
                this._zahl2 = value;
                // eine Änderung der Oberfläche veranlassen,
                // ist nur notwendig, wenn Sie die Felder
                // vom ViewModel aus auf 0 setzten wollen
                this.OnPropertyChanged("Zahl2");
            }
        }

        private string _zahlErgebnis;
        public string ZahlErgebnis
        {
            get { return this._zahlErgebnis; }
            set
            {
                this._zahlErgebnis = value;
                // eine Änderung der Oberfläche veranlassen
                this.OnPropertyChanged("ZahlErgebnis");
            }
        }

        // ist für die Ausgabe von Fehlermeldungen wichtig
        private string _fehler; 
        public string Fehler
        {
            get { return _fehler; }
            set
            {
                _fehler = value;
                // eine Änderung der Oberfläche veranlassen
                this.OnPropertyChanged("Fehler");
            }
        }

        // wichtig für die Befehle
        private bool _canExecute = true;
        public bool CanExecute
        {
            get { return this._canExecute; }
            set
            {
                if (this._canExecute == value)
                {
                    return;
                }
                this._canExecute = value;
            }
        }

        // wichtig für die Buttons
        private ICommand _additionButtonCommand;
        public ICommand AdditionButtonCommand
        {
            get { return _additionButtonCommand; }
            set { _additionButtonCommand = value; }
        }

        private ICommand _subtraktionButtonCommand;
        public ICommand SubtraktionButtonCommand
        {
            get { return _subtraktionButtonCommand; }
            set { _subtraktionButtonCommand = value; }
        }

        private ICommand _resetButtonCommand;
        public ICommand ResetButtonCommand
        {
            get { return _resetButtonCommand; }
            set { _resetButtonCommand = value; }
        }

        // wichtig für die Bindung zur Laufzeit
        public ViewRechnen Formular { get; set; }
        // <--

        #endregion

        #region Aktionen vom Anwender
        // Punkt 9.4 -->
        private void Addition(object obj)
        {
            try
            {
                Fehler = ""; // zurücksetzen
                // die Daten an das Model geben
                this._z.Zahl1 = Convert.ToDecimal(Zahl1);
                this._z.Zahl2 = Convert.ToDecimal(Zahl2);
                // hier die Methode Add aufrufen und das 
                // Ergebnis in einen String umwandeln
                ZahlErgebnis = this._z.Add().ToString();
                Farbe();
            }
            catch(Exception ex)
            {
                Fehler = "Fehler bei der Addition:"+ Environment.NewLine + ex.Message;
                ZahlErgebnis = "Fehler!";
            }
        }

        private void Subtraktion(object obj)
        {
            try
            {
                Fehler = ""; // zurücksetzen
                // die Daten an das Model geben
                this._z.Zahl1 = Convert.ToDecimal(Zahl1);
                this._z.Zahl2 = Convert.ToDecimal(Zahl2);
                // hier die Methode Sub aufrufen und das 
                // Ergebnis in einen String umwandeln
                ZahlErgebnis = this._z.Sub().ToString();
                Farbe();
            }
            catch (Exception ex)
            {
                Fehler = "Fehler bei der Subtraktion:" + Environment.NewLine + ex.Message;
                ZahlErgebnis = "Fehler!";
            }
        }

        private void Reset(object obj)
        {
            Zahl1 = "0";
            Zahl2 = "0";
            ZahlErgebnis = "0";
            // Farbe auf default
            FarbeVordergund = Brushes.Black;
        }
        // <--
        #endregion

        #region Hilfsmethoden
        // Hier ändern Sie die Vordergrundgrundfarbe
        private void Farbe()
        {
            // default mit weißem Hintergrund
            FarbeVordergund = Brushes.Black;
            if (this._z.ZahlErg > 0)
                FarbeVordergund = Brushes.Green;
            if (this._z.ZahlErg < 0)
                FarbeVordergund = Brushes.Red;
        }

        private SolidColorBrush _farbeVordergund;
        public SolidColorBrush FarbeVordergund
        {
            get { return _farbeVordergund; }
            set
            {
                _farbeVordergund = value;
                // eine Änderung der Oberfläche veranlassen
                this.OnPropertyChanged("FarbeVordergund");
            }
        }
		
        #endregion

        // der Konstruktor
        public RechnenViewModel()
        {
            // hier legen Sie die Methode für die Aktionen der Anwender fest, Punkt 9.4 -->
            _additionButtonCommand = new RelayCommand(Addition, param => this._canExecute);
            _subtraktionButtonCommand = new RelayCommand(Subtraktion, param => this._canExecute);
            _resetButtonCommand = new RelayCommand(Reset, param => this._canExecute);
            // <--
			
			_zahl1 = "0"; // das ist optional
			_zahl2 = "0";
			_zahlErgebnis = "0";
        }
    }
}
